import { Component, HTMLAttributes } from "react";
import { VersionMap } from "./content-type";
export declare type ComponentHighlighterProps = {
    /** enable highlighting */
    active?: boolean;
    /** show full scope name, including owner */
    fullScopeName?: boolean;
    /** override versions for specific components. As versions change rapidly consumers may want to specify them locally. */
    versionMap?: VersionMap;
    /** list ids to ignore */
    blacklist?: Set<string>;
    /** Actively recalculate position, to support moving elements  */
    motionTracking?: boolean;
} & HTMLAttributes<HTMLDivElement>;
declare type ComponentHighlighterState = {
    highlightTargetId?: string;
    targetElement?: HTMLElement;
};
/**
 * Identify components using a border and tooltip.
 * The floating elements are positioned using [Popper.js](https://popper.js.org/), with some custom code.
 *
 * To select an element in the page, we listen to 'mouseOver' event for any user interaction with a child element,
 * and then look for the first parent that has the html attribute "data-bit-id".
 * Components with the 'data-ignore-component-highlight' attribute are completely ignored,
 * to avoid changing target when interacting with the highlighter itself.
 * @name ComponentHighlighter
 */
export declare class ComponentHighlighter extends Component<ComponentHighlighterProps, ComponentHighlighterState> {
    state: ComponentHighlighterState;
    static defaultProps: {
        fullScopeName: boolean;
    };
    componentWillReceiveProps(nextProps: ComponentHighlighterProps): void;
    componentWillUnmount(): void;
    private _highlight;
    private highlight;
    private handleEnter;
    private handleLeave;
    render(): JSX.Element;
}
export {};
