"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComponentHighlighter = void 0;
const react_1 = __importStar(require("react"));
const clsx_1 = __importDefault(require("clsx"));
const lodash_debounce_1 = __importDefault(require("lodash.debounce"));
const styles = __importStar(require("./component-highlighter.module.scss"));
const base_ui_gfx_ref_tooltip_1 = require("@watheia/base-ui.gfx.ref-tooltip");
const baseUiUtilsTrackingOverlayBorder = __importStar(require("@watheia/tools.tracking.overlay-border"));
const tools_tracking_component_label_1 = require("@watheia/tools.tracking.component-label");
/**
 * Identify components using a border and tooltip.
 * The floating elements are positioned using [Popper.js](https://popper.js.org/), with some custom code.
 *
 * To select an element in the page, we listen to 'mouseOver' event for any user interaction with a child element,
 * and then look for the first parent that has the html attribute "data-bit-id".
 * Components with the 'data-ignore-component-highlight' attribute are completely ignored,
 * to avoid changing target when interacting with the highlighter itself.
 * @name ComponentHighlighter
 */
class ComponentHighlighter extends react_1.Component {
    constructor() {
        super(...arguments);
        this.state = {
            highlightTargetId: undefined,
            targetElement: undefined
        };
        this._highlight = (targetElement) => {
            const { blacklist = new Set() } = this.props;
            for (let elem = targetElement; elem; elem = elem.parentElement) {
                if (elem.hasAttribute("data-ignore-component-highlight")) {
                    return; //ignore
                }
                const value = elem.getAttribute("data-bit-id");
                if (value && blacklist.has(value)) {
                    continue; //skip
                }
                if (value) {
                    this.setState({ targetElement: elem, highlightTargetId: value });
                    return; //success!
                }
            }
            this.setState({ targetElement: undefined, highlightTargetId: undefined });
            return; //default - highlight none
        };
        this.highlight = (0, lodash_debounce_1.default)(this._highlight, 50);
        this.handleEnter = (event) => {
            const { target } = event;
            const element = target;
            if (!element)
                return;
            this.highlight(element);
        };
        this.handleLeave = (event) => {
            this.highlight(null);
        };
    }
    componentWillReceiveProps(nextProps) {
        const prevProps = this.props;
        if (prevProps.active !== nextProps.active && nextProps.active !== true) {
            this.setState({ highlightTargetId: undefined, targetElement: undefined });
        }
    }
    componentWillUnmount() {
        this.highlight.cancel();
    }
    render() {
        const _a = this.props, { active, children, fullScopeName, 
        // versionMap = {},
        blacklist, motionTracking } = _a, rest = __rest(_a, ["active", "children", "fullScopeName", "blacklist", "motionTracking"]);
        const { highlightTargetId, targetElement } = this.state;
        return (react_1.default.createElement("div", Object.assign({}, rest, { 
            // DO NOT ADD! use data-ignore in specific children
            // data-ignore-component-highlight
            className: (0, clsx_1.default)(styles.componentHighlighter, active && styles.active, highlightTargetId && styles.highlighting), 
            // "Both `onmouseenter` and `onmouseover` fire when the mouse enters the boundary of an element.
            // However, `onmouseenter` doesn't fire again (does not bubble),
            // if the mouse enters a child element within this first element."
            // https://stackoverflow.com/a/1638929/9941961
            // (this is perfect for this use case)
            onMouseOver: this.handleEnter, 
            // triggers when mouse exits this element (and not its children)
            onMouseLeave: this.handleLeave }),
            children,
            react_1.default.createElement(base_ui_gfx_ref_tooltip_1.RefTooltip, { className: styles.tooltip, targetElement: targetElement, motionTracking: motionTracking },
                react_1.default.createElement(tools_tracking_component_label_1.ComponentLabel, { bitId: highlightTargetId, fullScopeName: fullScopeName, "data-ignore-component-highlight": true })),
            react_1.default.createElement(baseUiUtilsTrackingOverlayBorder.OverlayBorder, { targetElement: targetElement, className: styles.border, motionTracking: motionTracking, "data-ignore-component-highlight": true })));
    }
}
exports.ComponentHighlighter = ComponentHighlighter;
ComponentHighlighter.defaultProps = {
    fullScopeName: true
};
//# sourceMappingURL=component-highlighter.js.map